package sun.security.util;

import java.util.Map;
import java.util.Arrays;

public abstract class Cache<K, V>
{
    protected Cache() {
    }
    
    public abstract int size();
    
    public abstract void clear();
    
    public abstract void put(K p0, V p1);
    
    public abstract V get(Object p0);
    
    public abstract void remove(Object p0);
    
    public abstract void setCapacity(int p0);
    
    public abstract void setTimeout(int p0);
    
    public abstract void accept(Cache.CacheVisitor<K, V> p0);
    
    public static <K, V> Cache<K, V> newSoftMemoryCache(int n) {
        return new MemoryCache(true, n);
    }
    
    public static <K, V> Cache<K, V> newSoftMemoryCache(int n, int n2) {
        return new MemoryCache(true, n, n2);
    }
    
    public static <K, V> Cache<K, V> newHardMemoryCache(int n) {
        return new MemoryCache(false, n);
    }
    
    public static <K, V> Cache<K, V> newNullCache() {
        return NullCache.INSTANCE;
    }
    
    public static <K, V> Cache<K, V> newHardMemoryCache(int n, int n2) {
        return new MemoryCache(false, n, n2);
    }
    
    public static class EqualByteArray
    {
        private final byte[] b;
        private volatile int hash;
        
        public EqualByteArray(byte[] b) {
            this.b = b;
        }
        
        @Override
        public int hashCode() {
            int hash = this.hash;
            if (hash == 0) {
                hash = this.b.length + 1;
                for (int i = 0; i < this.b.length; ++i) {
                    hash += (this.b[i] & 0xFF) * 37;
                }
                this.hash = hash;
            }
            return hash;
        }
        
        @Override
        public boolean equals(Object o) {
            return this == o || (o instanceof Cache.EqualByteArray && Arrays.equals(this.b, ((Cache.EqualByteArray)o).b));
        }
    }
    
    public interface CacheVisitor<K, V>
    {
        public abstract void visit(Map<K, V> p0);
    }
}
